@@ -6,15 +6,16 @@ residuals.scam <-function(object, type = c("deviance", "pearson","scaled.pearson
 { type <- match.arg(type)
   y <- object$y
   mu <- object$fitted.values
+  y.mu <- y-mu
 ##  family <- object$family
   wts <- object$prior.weights
   res<- switch(type,working = object$residuals,
-         scaled.pearson = (y-mu)*sqrt(wts)/sqrt(object$sig2*object$family$variance(mu)),
-              pearson = (y-mu)*sqrt(wts)/sqrt(object$family$variance(mu)),
-              deviance = { d.res<-sqrt(pmax(object$family$dev.resids(y,mu,wts),0))
+         scaled.pearson = y.mu*wts^.5/sqrt(object$sig2*object$family$variance(mu)),
+              pearson = y.mu*wts^.5/sqrt(object$family$variance(mu)),
+              deviance = { d.res <- sqrt(pmax(object$family$dev.resids(y,mu,wts),0))
                            ifelse(y>mu , d.res, -d.res)             
                          },
-              response = y - mu)
+              response = y.mu)
   res <- naresid(object$na.action,res)
   res
 }
